"""
Kplot is a vector based OO(Object Oriented) plot interface for Python.
It's aim is to provide an inuitive interface for making plots in the Python
language. Kplot uses current libraries like PGPLOT and Ghostscript, but
understanding of any of these programs is not required. Kplot can
output to screen, postscript file and several image file formats. It
runs on Windows and Linux(other posix compliant OS's should work too).
In combination with the Numeric package, which can perform array operations
(similar to Mathlab and IDL), Kplot is a powerfull tool for making plots.
"""
__doc__ +="""Kplot is developed at the Kapteyn Institute in Groningen, The Netherlands.
The authors of Kplot are Maarten Breddels and Edo Loenen. The Kplot
homepage can be found at www.astro.rug.nl/~kplot/."""

"""Kplot is an Object Oriented plot interface for Pyton.

I recommend using the pydoc webserver to view the documentation.
#pydoc -p <port>
will start the webserver, now you can browse to http://localhost:<port>/
where <port> is a portnumber, like 8001.

Kplot itself is a package, and has several subpackages:
-Kplot: main package
	-plotobjects: plottable objects, like Polygon, Histogram etc.
	-algo: algorithms like binning data for a Histogram.
	-points: shapes for the Point plottable objects, an X, +, tristar etc.
	-pgplot: an output device
	-ps: an output device
	-qt: an output device
	-simple: simplified plotting routines

-Kplot: main package
see the modules below, they are all imported into the Kplot namespace.
Like the color module has the class Color, you don't have to type
Kpot.color.Color, but Kplot.Color will do.


open the Python console by running 'python', and type alot with the '>>>' lines
 NOTE: when you see help(Kplot.<sth>) you can also use your browser if you run
 pydoc using as a webserver.

Using Kplot you start with making a Device.
the Kplot.pgplot.Device is a good Device.
>>> import Kplot
>>> dev = Kplot.pgplot.Device()
This will open the device and you'll see a window pop up.
A Device is very primitive to plot with.
>>> dev.plotPolyline(x=[0, 1], y=[0,1], color="red", linewidth=1, linestyle="normal", units="mm")
This will draw a line from (0, 0) to (1,1). But you probably don't want to draw
in a (0,0)-(1,1) range, with primitive commands like this, and no default
linewidth, linestyle etc.

Drawing a line can be easier.
>>> line = Kplot.plotobjects.Line(p1=(0,0), p2=(1,0.5))
>>> line.plot(dev)
I've just introduced a PlotObject, or a plottable object.
Kplot.plotobjects.Line is a PlotObject, it uses the primitive calls that Device has
to draw itself and chooses default values for you.
p1 and p2 are 2-dimensional points in the x,y form, they should be
a sequence of length 2, a tuple or list will do.
>>> help(Kplot.plotobjects.Line)
This will give some information about the PlotObject
the constructor(the __init__) is the method that gets called when you make
an object out of a class.
Now close the Device.
>>> dev.close()

Now open the Device again
>>> dev.open()
I'm gonna introduce a Panel now
>>> panel = dev.createPanel(p1=(0,0), p2=(0.75, 0.5))
You don't see anything happen on the screen yet.
A Panel is a container object, it can hold PlotObjects, and plot them on demand.
It also has a background, it choose a random color.
>>> panel.fillstyle
'solid'
>>> panel.color
<class Kplot.color.Color instance r=0.2824 g=0.3463 b=0.4513>
You will probaly get another color, because it is random.
>>> dev.output()
Now you'll see the your Panel's background
Make a line again, and add it to the panel
>>> line = Kplot.plotobjects.Line(p1=(0,0), p2=(1,1))
>>> panel.add(line)
Again you don't see anything, because the Panel will only plot(and replot)
itself when you ask for it by calling Device.output
>>> dev.output()
And you see the line

We'll set world coordinates now
>>> panel.setWorld(p1=(-5, -5), p2=(20, 20))
>>> dev.output()
The bottom left coordinate will be (-5, -5) and the top right coordinate
will be (20, 20).
The line is replotted, and is much smaller now, because (0,0) to (1,1) is
a small part in world coordinates.
>>> line.p2 = (20, 20)
>>> line.p1 = (0, -5)
>>> dev.output()
And the line will be larger

We'll explore units now
>>> dev.units
'mm'
The default units of the Device is in millimeters.
>>> line.linewidth = 10
>>> dev.output()
And you line will be 1 cm thick
Most plotobjects have a units attribute
>>> line.units = "cm"
>>> line.linewidth = 0.5
>>> dev.output()
And your line will be 0.5 centimeter thick
>>> help(Kplot.Device)
Here you'll find the available units ("normalised", "inch", "points", "mm" and
"cm" probably
>>> line.info()
This displays all information for this line

Run this again
>>> help(Kplot.plotobjects.Line)
You see attributes, like:
	p1 = attribute
		default: no default
		allowed types: any type
		info: a 2 dimensional point in the form of a (x, y) tuple
and
	linewidth = attribute
		default: 0.5
		allowed types: number
		info: no info
p1 is required in the constructor, as is p2. But others, like linewidth
aren't. They can have default values, but can also be set in the constructor.

>>> line2 = Kplot.Line(p1=(0,0), p2=(10,0), linewidth=2, color="white")
>>> panel.add(line2)
>>> dev.output()
And a white line is visible, 2mm thick


To get information about the methods you called, browse the documentation

this is what you can find in the Kplot package:
vector: Vector(class)
matrix: Matrix(class)
color: Color(class)
colormap: ColorMap(class)
device: Device(class), CapsException(class/Exception)
deviceabstract: DeviceAbstract(class for use for developers only)
panel: Panel(class)
tools: toradians(function), todegrees(function)


Line is part of the plotobjects package, that's why you typed:
Kplot.plotobjects.Line
but because plotobject is such a long name, and PlotObjects are used alot,
the whole plotobject package is imported into the Kplot namespace.
Kplot.Line will also do.


Look at the Kplot.plotobjects package to learn what PlotObjects there are.
You'll get familair with other parts of Kplot when you see the examples
provided in that package.

Look at the Kplot.device, and Kplot.panel for documentation about
the methods you used.

The Vector and Matrix class in this package are mainly used internally,
but can be of use to you.

There are 2 methods of plotting:
	Kplot.createDevice()
	      this creates the default X11 screen output
	Kplot.hardcopy(filename,filetype,orientation)
	      this writes a hardcopy of the Kplot.createDevice().
	      - filetype can be 'gif','ps' (black and white postscript),
	      'cps' (color postscript)
	      - orientation can be 'portrait'(default) or 'landscape'

	      if filetype and orientation are nor set, the hardcopy
	      is used which is specified in your KPLOT_HARDCOPY_DEVICE
	      environment


Good luck

"""

#__all__ = ["vector", "matrix"]

version = "Kplot-1"

if not __builtins__.has_key("True"):
	__builtins__["True"] = 1
if not __builtins__.has_key("False"):
	__builtins__["False"] = 0

from vector import Vector
from matrix import Matrix
from color import Color
from colormap import ColorMap

from deviceabstract import DeviceAbstract
from device import Device, CapsException
from tools import toradians, todegrees
from panel import Panel


import os
import ps

global _defaultDeviceClassName
try:
	import ppgplot
except ImportError:
	_defaultDeviceClassname = "Kplot.ps.WindowDevice"
	_defaultHardcopyDeviceClassname = "Kplot.ps.FileDevice"
else:
	import pgplot # PGPLOT device
	_defaultDeviceClassname = "Kplot.pgplot.WindowDevice"
	_defaultHardcopyDeviceClassname = "Kplot.pgplot.PsPortraitDevice"
	del ppgplot


import Kplot

def createDevice(**kwargs):
	"""Creates a default device

	Keyword Arguments:
	size -- a (width, height) tuple for the dimension of the device.
					(in units, default is "mm", or as argument)
	units -- the units in which the size is given.
	"""

	className = os.environ.get("KPLOT_DEVICE", _defaultDeviceClassname)
	devSize = os.environ.get("KPLOT_SIZE")
	import Kplot
	dir(Kplot)
	try:
		exec "devclass = " +className
	except:
		print "!!! your KPLOT_DEVICE variable may be set incorrectly, try Kplot.pgplot.WindowDevice or Kplot.ps.WindowDevice"
		raise
	if devSize != None and ("size" not in kwargs):
		exec "size = " +devSize
		kwargs["size"]  = size

	dev = apply(devclass, [], kwargs)
	return dev

def createHardcopyDevice(filename=None, filetype=None, orientation="portrait", **kwargs):
	"""Makes a default hardcopy device, don't add extension to filename"""
	orientations = {"portrait" : 0,
			"landscape": 1
			}
	
	devices = {"gif":("Kplot.pgplot.GifPortraitDevice", "Kplot.pgplot.GifLandscapeDevice"),
		"cps":("Kplot.pgplot.PsPortraitDevice","Kplot.pgplot.PsLandscapeDevice"),
		"ps" :("Kplot.pgplot.PsPortraitDeviceBw","Kplot.pgplot.PsLandscapeDeviceBw"),
		}
	
	if filetype <> None:
		if filetype.lower() in devices:
			className = (devices[filetype.lower()])[orientations[orientation.lower()]]
		else:
			raise NameError, ">>>"+filetype+"<<< : Unknow hardcopy device. Use: "+str(devices) 
	else:
		className = os.environ.get("KPLOT_HARDCOPY_DEVICE", _defaultHardcopyDeviceClassname)
	
	devSize = os.environ.get("KPLOT_HARDCOPY_SIZE")
	import Kplot
	try:
		exec "devclass = " +className
	except:
		print "!!! your KPLOT_HARDCOPY_DEVICE variable may be set incorrectly, try Kplot.pgplot.PsLandscapeDevice or Kplot.ps.FileDevice"
		raise
	if devSize != None and ("size" not in kwargs):
		exec "size = " +devSize
		kwargs["size"]  = size

	kwargs["filename"] = filename

	dev = apply(devclass, [], kwargs)
	return dev

def makeTestDevice():
	"""Makes a test device for unit tests, shouldn't generate output"""
	return pgplot.Device("/NULL")




import plotobjects
# we don't wanna type plotobjects.Line all the time, right ?
from plotobjects import *
#__path__.append(__path__[0] +"/plotobjects")

import points
markertype = points
#import qt # Qt Device
#import ps # Postscript device

import simple
import algo
import quick

import os
if os.environ.has_key("KPLOT_STARTUP"):
	execfile(os.environ["KPLOT_STARTUP"])
