"""An image uses a colormap to map it's value to a color.

Colormaps should be callable objects, like a function, lambda, or an object
with the __call__ method.

If you want to make you own colormap in Kplot you can just create a function.
	def totalwhite(value):
		return Color.white

or a lambda object:
	lambda value: return Color(value, 0, 0)

or a class with the __call__ method:
	class SharpEdgeColorMap:
		def __init__(self, colorlist):
			self.colorlist = colorlist
		def __call__(self,  value):
			length = len(self.colorlist)
			index = int((value * (length-1)))
			return self.colorlist[index]

the colormap will be called with values between 0 and 1. Where 0 should map
to the color of the lowest value, and 1 should map to the color with the highest
value.

Use the ColorMap class to make colormaps that interpolate between colors

"""

from tools import abstract
from color import Color
import math
from Numeric import *

step = 1/256.
offset = .05

def interpol(x,function,colors,allvalues):
	x = x+offset
	position = eval(function)
	minpos = min(allvalues)
	maxpos = max(allvalues)
	position = (position - minpos)/(maxpos-minpos)
	return position

class ColorMap:
	"""A colormap that interpolates between colors in a list.
	The interpolation function can be given as a string (must be function of x
	e.g. 'x**2' or 'log10(x)')
	Default function is 'x'
	If function is set to 'sharp', a colormap is created without interpolation
	(Ron Ekers)

	example:
		colormap = ColorMap([Color.red, Color.white, Color.green])
	With this colormap, the minimum intensity maps to red, the maximum to
	green. The mean maps to white, and other values are linear interpolated.

	There are some predefined ColorMaps
		ColorMap.rainbow:
			purple-blue-cyan-green-yellow-red
		ColorMap.heat:
			black-red-orange-white
		ColorMap.cool:
			black-blue-white
		ColorMap.blackwhite:
			black-white
		ColorMap.whiteblack:
			white-black
	"""


	def __init__(self, colors, function='x'):
		"""Constructor

		Arguments:
		colors   -- A sequence of Kplot.Color objects
		function -- interpolation function between the colors.
		            give function as string. should be function of 'x'
			    e.g. 'x**2' or 'log10(x)'
			    Default function is 'x'
			    If function is set to 'sharp', a colormap is created
			    without interpolation(Ron Ekers)
		"""
		self.colors = colors
		self.function = function

	def __call__(self, position):
		length = float(len(self.colors))
		colors = self.colors
		if self.function == 'sharp':
			position = math.floor((position*length)*(1-1E-6))/(length-1)
		else:
			x = array((arange(0,1+step,step))+offset)
			allvalues = eval(self.function)
			position = interpol(position,self.function,self.colors,allvalues)
		colindex1 = int(math.floor((float(position) * (length-1))))
		colindex2 = int(math.ceil((float(position) * (length-1))))
		color1 = self.colors[colindex1]
		color2 = self.colors[colindex2]
		rdiff = color2.r - color1.r
		gdiff = color2.g - color1.g
		bdiff = color2.b - color1.b
		relposition = (position - colindex1 / (length-1)) * (length-1)
		color = Color(color1.r + rdiff*relposition, color1.g + gdiff*relposition, color1.b + bdiff*relposition)
		return color

ColorMap.rainbow = ColorMap([Color.purple, Color.blue, Color.cyan, Color.green, Color.yellow, Color.red])
ColorMap.heat = ColorMap([Color.black, Color.red, Color.orange, Color.white])
ColorMap.cool = ColorMap([Color.black, Color.blue, Color.white])
ColorMap.blackwhite = ColorMap([Color.black, Color.white])
ColorMap.whiteblack = ColorMap([Color.white, Color.black])
