"""A container class for PlotObjects

Panels are used to contain PlotObjects and mangage world coordinates
The world coordinates are set using Panel.setWorld
PlotObjects are added using Panel.add, and removed with Panel.remove

Your first Panel starts with a Device.createPanel call, you can call this
method multiple times, to create more panels, for making overlays etc.
But Panel can also make new Panels.

>>> import Kplot
>>> dev = Kplot.createDevice()
>>> panel = dev.createPanel()
>>> dev.output()

the arguments to createPanel can be found at Kplot.Device.createPanel

you can make mosiacs, or split panels into 2 child panels
>>> panelleft,  panelright = panel.split(0.2, "vertical")
>>> dev.output()
>>> mosaic = panelright.mosaic(2,3)
>>> dev.output()

Now every panel has it's own world coordinates, starting with the
default of p1=(0,0), p2=(1,1)

Panels get a random background color, you can set the Panel.color
attribute, or set the Panel.fillstyle to "nofill"

>>> panel.color = "white"
>>> mosaic[1][2].fillstyle = "nofill"
>>> dev.output()


You can get and set the viewport of a Panel, viewport coordinates
are in normalised coordinates relative to it's parent

>>> panelleft.getViewport()
(Vector(0.05, 0.05), Vector(0.15, 0.95))
>>> panelleft.setViewport(p1=(0,0), p2=(0.20, 0.20))
>>> dev.output()

Panels can be copied(or call it cloning)
>>> leftcopy = panelleft.copy()
>>> dev.output()

and overlays can be made, same as a copy, but fillstyle is set to "nofill",
so that it doesn't paints over the panel under it, and it's own world
coordinates.
>>> overlay = panel.overlay(p1=(-100, -10), p2=(200, 40)

Note that when a mosaic,copy,split or overlay is made, they are just normal
Panels. Just lik ethe Panel your started with. They all got their own world
coordinates. This can be usefull for graphs and histograms. You draw you graph
in one Panel, make an overlay and set the world coordinates the way you like,
and add a Box or Axis on the overlay.

>>> import Kplot
>>> dev = Kplot.createDevice()
>>> panel = dev.createPanel()
>>> panel.setWorld(p1=(0,0), p2=(9,9))
>>> scatter = Kplot.Scatter(range(10), [7,0,3,7,9,4,2,4,7,9], Kplot.markertype.x)
>>> panel.add(scatter)
>>> dev.output()
>>> overlay = panel.overlay(p1=(-100,-100), p2=(300,500))
>>> overlay.add(Kplot.Box())
>>> dev.output()

The Box works with the overlay's world coordinates, and the Scatter works
with the original (0,0)-(9,9) world coordinates
"""
from matrix import Matrix
from color import Color
from vector import Vector
import random
import Numeric
from weakref import proxy
from tools import issequence

class Panel:
	def __init__(self, p1, p2, parent=None):
		"""Constructor

		A Panel is a container for plottable objects.
		When a panel is plotted, it draws it's region in it's
		fillstyle/color, then draws all plottable objects, and at
		last it's child Panels.

		All objects are plotted with the Panel's worldmatrix, and are clipped
		agains the Panel, when Panel.clipping is true.


		Arguments:
		p1 -- bottom left point in the form of a tuple
		p2 -- top right point in the form of a tuple

		Optional Arguments:
		parent -- parent object, can be a Panel or a Device

		Attributes:
		parent -- parent object, see above
		children -- child objects, e.g. when making a mosaic of a Panel
		objects -- a list of plot objects(having a 'plot' method)
		clipping -- boolean, if this Panel clips plotobjects outside the Panel.
		viewportmatrix -- a Kplot.Matrix that transforms the unit square
			of this Panel to the parent's normalised coordinates
		worldmatrix -- a Kplot.Matrix that transforms the world coordinates
			to the unit square of this Panel
		fillstyle -- fillstyle for filling this Panel. default is "solid"
		color -- color for filling this Panel. default is a random color

		"""
		self.parent = parent
		self.children = []
		self.objects = []
		self.clipping = True
		#self.clipchildren = 1
		#self.clipparent = 0
		self.viewportmatrix = None
		self.worldmatrix = Matrix()

		self.setViewport(p1, p2)
		from plotobjects.nonbasic import Rectangle
		self.background = Rectangle(p1=(0,0), p2=(1,1), fillstyle="nofill", linestyle="noline",color="background")
		#self.fillstyle = "nofill"#"solid"
		#self.color = Color(random.random(), random.random(), random.random())
		#self.color = "foreground"
		
	def add(self, *objects):
		"""Add a plottable object, or a sequence of it

		A plottable object has a plot method, that takes a device as argument.
		example:
		class MyObject:
			def plot(self, dev):
				# plot code

		Arguments:
		objects -- a plottable object, or a sequence of it
		"""
		for o in objects:
			if issequence(o):
				for element in o:
					self.add(element)
			elif hasattr(o, "plot"):
				self.objects.append(o)
				o.addChangeListener(self)
			else:
				raise TypeError, "Can only add objects with plot method, or a sequence of these objects"

	def remove(self, *objects):
		"""Removes a plottable object, or a sequence of it"""
		for o in objects:
			if issequence(o):
				for element in o:
					self.remove(element)
			else:
				self.objects.remove(o)
				o.removeChangeListener(self)

	def setWorld(self, p1, p2):
		"""Set the worldmatrix so that the region of p1-p2 get displayed

		panel.setWorld(p1=(-10, -10), p2=(30, 40))
		world coordinate (-10, 10) will be at the bottom left of the panel,
		(30, 40) at the top right

		The width and height shouldn't be zero.

		Arguments:
		p1 -- the world coordinate for the bottom left corner
		p1 -- the world coordinate for the top right corner
		"""
		x1, y1 = p1
		x2, y2 = p2
		self.worldmatrix = (Matrix.translate(p1) * Matrix.scaleXY(x2-x1, y2-y1)).inverse()


	def setWorldXY(self, x, y, margin=0.0, xmargin=0.0, ymargin=0.0):
		"""Sets the worldmatrix so that the x and y sequences will fit on this panel

		The margins are the space around this x,y range, e.g. when plotting a function
		so that there is some space around the graph

		It is equal to:
			p1 = min(x)-margin-xmargin, min(y)-margin-ymargin
			p2 = max(x)+margin+xmargin, max(y)+margin+ymargin
			self.setWorld(p1, p2)
		Arguments:
		x -- a sequence of x elements
		y -- a sequence of y elements
		"""
		p1 = min(x)-margin-xmargin, min(y)-margin-ymargin
		p2 = max(x)+margin+xmargin, max(y)+margin+ymargin
		self.setWorld(p1, p2)

	def setWorldImage(self, imag, margin=0.0, xmargin=0.0, ymargin=0.0):
		"""Sets the worldmatrix so that the image(2 dim. array) will fit on this panel
		Which is equal to:
			p1 = 0.5, 0.5
			p2 = 0.5+len(data), 0.5+len(data[0])
			self.setWorld(p1, p2)

		The margins are the space around this x,y range, e.g. when plotting a function
		so that there is some space around the graph

		setWorldImage also shifts the axes as defined in in imag.origin.
		this points to the pixel which is (0,0) in world coordinates.

		Arguments:
		imag    - a Kplot.Image
		optional arguments:
		margin  - a float or integer indicates a margin in both x and y direction round the image
		xmargin - a float or integer indicates a margin in x direction round the image
		ymargin - a float or integer indicates a margin in y direction round the image
		"""
		data = imag.data
		(delx,dely) = imag.origin
		delx += .5
		dely += .5
		imag.origin = delx,dely
		p1 = 0.5-margin-xmargin-delx, 0.5-margin-ymargin-dely
		p2 = 0.5+margin+xmargin+len(data[0])-delx, 0.5+margin+ymargin+len(data)-dely
		self.setWorld(p1, p2)

	def getWorld(self):
		"""Returns the world coordinate of the bottom left corner as 2 tuples"""
		worldinv = self.worldmatrix.inverse()
		p1 = worldinv * (0, 0)
		p2 = worldinv * (1, 1)
		return (p1, p2)


	def setViewport(self, p1, p2):
		"""Sets the viewport as in the constructor"""
		x1, y1 = p1
		x2, y2 = p2
		self.viewportmatrix = Matrix.translate(p1) * Matrix.scaleXY(x2-x1, y2-y1)

	def getViewport(self):
		"""Returns the viewport corners in it's parents normalised device coordinates"""
		p1 = self.viewportmatrix * (0, 0)
		p2 = self.viewportmatrix * (1, 1)
		return (p1, p2)


	def plot(self, dev):
		"""Plot all objects and subpanels on a Device.

		First draws the background using Panel.color/fillstyle, then the
		plottable objects are plotted. And at last the subpanels(Panel.children).

		The subpanels are not clipped against it's parent Panel

		Arguments:
		dev -- an object of a Kplot.Device subclass.
		"""
		dev.setClipping(self.clipping)
		dev.pushViewportMatrix(self.viewportmatrix)

		#p1 = (0.0, 0.0)
		#p2 = (1.0, 1.0)
		#(x1, y1), (x2, y2) = (p1, p2)
		#x = Numeric.array([x1, x2, x2, x1])
		#y = Numeric.array([y1, y1, y2, y2])
		#dev.plotPolygon(x, y, color=self.color, linestyle="normal", linewidth=0, fillstyle=self.fillstyle)
		self.background.plot(dev)
		dev.pushWorldMatrix(self.worldmatrix)
		for o in self.objects:
			o.plot(dev)
		dev.popWorldMatrix()

		for child in self.children:
			child.plot(dev)
		dev.popViewportMatrix()


	def printObjects(self, indent=0):
		"""Print a list of all objects on stdout"""
		for object in self.objects:
			objectname = "plotobject: " +object.__class__.__name__
			print "    "*indent + "-", objectname

	def printMembers(self, indent=0):
		"""Prints a list of all subpanels and it's objects on stdout"""
		print "    "*indent, self.__module__ +"." +self.__class__.__name__
		self.printObjects(indent+1)
		for child in self.children:
			child.printMembers(indent+1)



	def getPos(self, clip=1):
		validrange = 0
		while not validrange:
			pos = self.parent.getPos(clip)
			if pos is None:
				return None
			pos = self.viewportmatrix.inverse() * pos
			if pos.x >= 0 and pos.x <= 1 and pos.y >= 0 and pos.y <= 1 or not clip:
				validrange = 1
		return pos

	def getWorldPos(self, clip=1):
		pos = self.getPos(clip)
		if pos is None:
			return None
		pos = self.worldmatrix.inverse() * pos

		return pos

	def createPanel(self, p1=(0.1, 0.1), p2=(0.9, 0.9), parent=None):
		"""Returns a new Panel that is a child of this Panel.

		Arguments:
		p1 -- the bottom left point
		p2 -- the top right point
		parent -- don't use this, it's used internally
		"""
		if parent == None:
			parent = self
		panel = self.parent.createPanel(p1=p1, p2=p2, parent=parent)
		if parent == self:
			self.children.append(panel)
		return panel

	def mosaic(self, width, height, spacing=0.05, parent=None):
		"""Return an sequence of Panels that form a width*height mosaic

		When width and height are both > 1, the returned sequence is 2
		dimensional.

		The sequence returned is 1-dimensional if width or height is 1.
		Otherwise it's 2-dimensional in column-row format.

		Arguments:
		width -- The number of horizontal Panels.
		height -- The number of vertical Panels.
		Optional arguments:
		spacing -- the spacing around each Panel, percentage of width and height
			of the current Panel. Between two panels there is
			2*spacing*width(or height) spacing.
		"""
		if parent == None:
			parent = self
		if not (width > 1 or height > 1):
			raise Exception, "can only make mozaics with width or height > 1"
		mosaics = []
		v1 = self.viewportmatrix * (0, 0)
		v2 = self.viewportmatrix * (1, 1)
		v1x, v1y = (0.0, 0.0)#v1
		v2x, v2y = (1.0, 1.0)#v2

		viewportwidth = v2x - v1x
		viewportheight = v2y - v1y
		for x in range(0, width):
			for y in range(0, height):
				x1 = v1x + (float(x) / width) * viewportwidth + spacing * viewportwidth
				x2 = v1x + (float(x+1) / width) * viewportwidth - spacing * viewportwidth
				y1 = v1y + (float(y) / height) * viewportheight + spacing * viewportheight
				y2 = v1y + (float(y+1) / height) * viewportheight - spacing * viewportheight
				p1 = (x1, y1)
				p2 = (x2, y2)
				mosaics.append(self.createPanel(p1=p1, p2=p2, parent=parent))
		#for panel in mosaics.
		if width == 1 or height == 1:
			mosaicarray = Numeric.array(mosaics)
		else:
			mosaicarray = Numeric.reshape(Numeric.array(mosaics), (width, height))
		return mosaicarray


	def split(self, middle, how, spacing=0.05, parent=None):
		"""Creates two Panels which are a split from this Panel, returned as a list.

		This split does not 'destroy' the original Panel. The Panels created
		overlap the original Panel.

		When split horizontal, the first element of the list returned contains the
		bottom part of the split, the second element the top part of the split.

		When split vertical, the first element of the list returned contains the
		left part of the split, the second element the right part of the split.

		For middle = 0.5, it will split the Panel in two equal sized Panels.
		A value of 0.2 will split it in a Panel of 0.2 times the width or height
		of the original Panel(minus the spacing), and a Panel of 0.8 times the
		width or height(depending if it's split vertical or horizontal).

		e.g:
		panelBottom, panelTop = panel.split(0.5, "vertical")

		Arguments:
		middle -- A float between 0 and 1 that refers to the middle of the split.
		how -- The way to split. "horizontal" or "vertical".
		spacing -- The spacing between the Panels. relative to the width or height

		"""
		if how == "vertical":
			v1 = self.viewportmatrix * (0, 0)
			v2 = self.viewportmatrix * (1, 1)
			v1x, v1y = (0.0, 0.0)#v1
			v2x, v2y = (1.0, 1.0)#v2
			width = v2x - v1x
			x1a = v1x + spacing * width
			x2a = v1x + width * middle - (spacing * width)
			x1b = v1x + width * middle + (spacing * width)
			x2b = v2x - spacing * width
			y1 = v1y + spacing * width
			y2 = v2y - spacing * width

			#print (x1a, y1), (x2a, y2)
			return [self.createPanel(p1=(x1a, y1), p2=(x2a, y2), parent=parent), self.createPanel(p1=(x1b, y1), p2=(x2b, y2), parent=parent)]
		elif how == "horizontal":
			v1 = self.viewportmatrix * (0, 0)
			v2 = self.viewportmatrix * (1, 1)
			v1x, v1y = (0.0, 0.0)
			v2x, v2y = (1.0, 1.0)
			height = v2y - v1y
			x1 = v1x + spacing * height
			x2 = v2x - spacing * height
			y1a = v1y + spacing * height
			y2a = v1y + height * middle - (spacing * height)
			y1b = v1y + height * middle + (spacing * height)
			y2b = v2y - spacing * height
			#print "y:", self.__vy1, y1, y2, self.__vy2
			return [self.createPanel(p1=(x1, y1a), p2=(x2, y2a), parent=parent),
					self.createPanel(p1=(x1, y1b), p2=(x2, y2b), parent=parent)]
			#return [self.createPanel(((self.__vx1, self.__vy1),( self.__vx2, y1)), self.__device), self.createPanel(((self.__vx1, y2), (self.__vx2, self.__vy2)), self.__device)]
		else:
			raise Exception, "choose 'vertical' or 'horizontal' for split, not: " +str(how)

	def overlay(self, p1, p2):
		"""Returns a transparant copy of the Panel with world coordinates p1,p2"""
		panel = self.copy()
		panel.background.fillstyle = "nofill"
		panel.setWorld(p1=p1, p2=p2)
		return panel

	def copy(self):
		"""Return a copy of the Panel"""
		panel = self.createPanel(parent=self.parent)
		panel.viewportmatrix = Matrix(self.viewportmatrix)
		panel.worldmatrix = Matrix(self.worldmatrix)
		panel.background.color = self.background.color
		panel.background.fillstyle = self.background.fillstyle
		panel.background.linestyle = self.background.linestyle
		panel.clipping = self.clipping
		return panel

	def fromWorld(self, pos):
		matrix = self.viewportmatrix * self.worldmatrix
		parent = self.parent
		while isinstance(parent, Panel):
			matrix = parent.viewportmatrix * matrix
			parent = parent.parent
		return matrix * pos

	def toWorld(self, pos):
		matrix = self.viewportmatrix * self.worldmatrix
		parent = self.parent
		while isinstance(parent, Panel):
			matrix = parent.viewportmatrix * matrix
			parent = parent.parent
		return matrix.inverse() * pos


	def changeNotify(self):
		if self.parent:
			self.parent.changeNotify();

	def getClipping(self):
		return self.clipping

	def setClipping(self, enabled):
		self.clipping = enabled
