"""Contains MarkerLine class"""
from plotobject import PlotObject
from basic import Polyline
from nonbasic import Markers
from properties import *
from autoproperty import *


class MarkerLine(PlotObject):
	"""MarkerLine is a compound object of Markers and Polyline

	MarkerLine will draw Markers, and a Polyline between them when connect is True.

	See the help of Markers and Polyline for more help.

	Properties:
	- polyline
	- markers 
	- connect 
	- matrix
	- x
	- y
	"""
	polyline = autoproperty(typelist = [Polyline], docstring="The Polyline object that will be plotted if connect is true")
	markers = autoproperty(typelist = [Markers], docstring="The Markers object that will be plotted")
	connect = booleanproperty(default=True, docstring="If True, a Polyline will be plotted through the markers")
	matrix = matrixproperty()
	x = arrayproperty()
	y = arrayproperty()

	def __init__(self, x, y, markertype, **properties):
		"""Will create a Markers and Polyline based on the arguments

		Arguments:
		x -- A Numeric array containing all x elements.
		y -- Idem for y.
		markertype -- see Marker

		Optional arguments:
		All optional arguments will be passed to Polyline and Markers as default.
		"""
		PlotObject.__init__(self, properties)
		self.polyline = Polyline(x, y, defaults=properties)
		self.polyline.addChangeListener(self)
		self.markers = Markers(x, y, markertype, defaults=properties)
		self.markers.addChangeListener(self)
		self.x = x
		self.y = y

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		self.polyline.x = self.x
		self.polyline.y = self.y
		self.markers.x = self.x
		self.markers.y = self.y
		if self.connect:
			self.polyline.plot(dev)
		self.markers.plot(dev)
		dev.popWorldMatrix()


	def example():
		import Kplot
		import Numeric
		import RandomArray
		print "MarkerLine example"
		print
		print 'Like Polyline and Markers, is the input of MarkerLine'
		print 'two arrays, one with x coordinates and one with y.'
		print
		print 'x = Numeric.arange(0, 10.5, 0.5)'
		print 'y = x ** 2 + RandomArray.random(len(x)) * 10'
		print 'scatter = Kplot.MarkerLine(x, y, Kplot.markertype.solidcircle)'
		x = Numeric.arange(0, 10.5, 0.5)
		y = x ** 2 + RandomArray.random(len(x)) * 10
		scatter = Kplot.MarkerLine(x, y, Kplot.markertype.solidcircle)
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(scatter)
		panel.setWorld(p1=(-1, -1), p2=(11, 111))
		dev.output()
		dev.close()

	example = staticmethod(example)
