"""Histogram"""
from plotobject import *
from basic import *
from properties import *
from autoproperty import *
import Numeric
import math

class Histogram(PlotObject):
	"""Draws binned data

	Use Kplot.algo.bindata to bin the data.

	The bins will be drawn between [0, len(bindata)] for x, and
	[0, max(bindata)] for y.

	To set world coordinates to make a histogram fit:
	p1 = (0, 0)
	p2 = (len(bindata), max(bindata) * 1.03) # *1.03 for some spacing above
	panel.setWorld(p1=p1, p2=p2)


	The Histogram can be transformed by it's matrix to make it fit on the panel
	so that the Axis is labeled like you want.

	Properties:
        - color
        - linewidth
        - linestyle
        - fillstyle
        - matrix 
        - bindata
        - units
        - polyline
	  (if True, instead of a polygon, a polyline is drawn, so the bottom line is not drawn)

	"""
	color = colorproperty()
	linewidth = linewidthproperty()
	linestyle = linestyleproperty()
	fillstyle = fillstyleproperty()
	matrix = matrixproperty()
	bindata = arrayproperty()
	units = unitsproperty()
	polyline = booleanproperty(default=False, docstring="if True, instead of a polygon, a polyline is drawn, so the bottom line is not drawn")

	def __init__(self, bindata, defaults=None, **properties):
		"""Constructor

		Arguments:
		bindata -- a sequence like a list, tuple or Numeric array containing
			the binned data, thus the height of the bin.
		"""
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.bindata = bindata

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)

		length = len(self.bindata)
		x = Numeric.repeat(range(length+1), (length+1)*[2])
		y = Numeric.repeat(self.bindata, length*[2])
		y = Numeric.concatenate(([0], y, [0]))
		if self.polyline:
			dev.plotPolyline(x, y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth, units=self.units)
		else:
			dev.plotPolygon(x, y, color=self.color, fillstyle=self.fillstyle, linestyle=self.linestyle, linewidth=self.linewidth, units=self.units)

		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Histogram example"
		print
		print 'Histogram draw a histogram of an array'
		print 'containing the number of elements in each bin'
		print 'This array can be made by hand, or by binning data using Kplot.algo.bindata.'
		print
		print 'example:'
		print 'binned = [1,5,7,2,4,9,6,2,5,8]'
		print 'hist = Kplot.Histogram(binned)'
		print
		print 'For properties see help.'

		binned = [1,5,7,2,4,9,6,2,5,8]
		hist = Kplot.Histogram(binned)

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(hist)
		panel.setWorld(p1=(0, 0), p2=(len(binned), max(binned)*1.1))
		dev.output()
		dev.close()

	example = staticmethod(example)

	def test():
		import Kplot

		print "the bottom right should have no baseline"



		binned = [1,5,7,2,4,9,6,2,5,8]
		hist0 = Kplot.Histogram(binned, fillstyle="solid")
		hist1 = Kplot.Histogram(binned, fillstyle="nofill")
		hist2 = Kplot.Histogram(binned, fillstyle="crosshatched")
		hist3 = Kplot.Histogram(binned, polyline=True)

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panels = panel.mosaic(2,2)
		hist0.matrix = Matrix.scaleXY(1.0 / len(binned), 1.0 / (max(binned)*1.1))
		panels[0][0].add(hist0)

		hist1.matrix = Matrix.scaleXY(0.5, 1)
		panels[0][1].setWorld(p1=(0, 0), p2=(len(binned)/2, max(binned) * 1.1))
		panels[0][1].add(hist1)

		panels[1][1].setWorld(p1=(0, 0), p2=(len(binned), max(binned) * 1.1))
		panels[1][1].add(hist2)
		panels[1][0].setWorld(p1=(0, 0), p2=(len(binned), max(binned) * 1.1))
		panels[1][0].add(hist3)
		dev.output()
		dev.close()

	test = staticmethod(test)




