"""Used to display what value a color in an image has"""
from plotobject import PlotObject
from properties import *
from autoproperty import *
from Kplot import toradians, createDevice
from basic import Image, Text
import Numeric
from box import Box

class Wedge(PlotObject):
	"""Used to display what value a color in an image has.  Minimum input
	needed is an Kplot.Image. Further the direction of filling can be
	given as second property (default = up), the side which is label
	(default = left), and whether the wedge should be symmetric around zero
	(default = None).
	If labeling is set to None, no axis will be labelled.
	examples:
	   Wedge(image)                            - wedge filled upward
	                                             labeling on lefthand side
						     
	   Wedge(image,'down')                     - wedge filled downward
	                                             labeling on lefthand side
						     
	   Wedge(image,'down','right')             - wedge filled downward
	                                             labeling on righthand side
						     
	   Wedge(image,'down','right','symmetric') - wedge filled downward
	                                             symmetric around zero
	                                             labeling on righthand side
						     
	   Wedge(image,'down',None)                - wedge filled downward
	                                             no labeling
	Properties:
        - image
        - matrix
        - direction 
        - labeling
        - sym
        - colors
        - box
        - labelstyle
        - tickstyle 
	"""
	image = autoproperty(typelist = [Image])
	matrix = matrixproperty()
	direction = stringproperty()
	labeling = stringproperty()
	sym = stringproperty()
	colors = numberproperty(default=256)
	box = autoproperty()
	labelstyle = stringproperty(default='normal')
	tickstyle = stringproperty(default='normal')

	def __init__(self, image, direction='up', labeling='left',sym=None ,defaults=None, **properties):
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.image = image
		self.direction = direction
		self.labeling = labeling
		self.sym = sym
		self.box = Box()
		self.box.leftlabeling = 'none'
		self.box.rightlabeling = 'none'
		self.box.toplabeling = 'none'
		self.box.bottomlabeling = 'none'
		self.box.leftticks = 'none'
		self.box.rightticks = 'none'
		self.box.topticks = 'none'
		self.box.bottomticks = 'none'
		self.box.drawgrid = False

		if self.sym == "symetric":
			self.sym = "symmetric"

		if self.labeling == "None":
			self.labeling = None
		
		if self.labeling == "left":
			self.box.leftlabeling = self.labelstyle.lower()
			self.box.leftticks = self.tickstyle.lower()
			self.box.rightticks = self.tickstyle.lower()
		elif self.labeling == "right":
			self.box.rightlabeling = self.labelstyle.lower()
			self.box.leftticks = self.tickstyle.lower()
			self.box.rightticks = self.tickstyle.lower()
		elif self.labeling == "top":
			self.box.toplabeling = self.labelstyle.lower()
			self.box.topticks = self.tickstyle.lower()
			self.box.bottomticks = self.tickstyle.lower()
		elif self.labeling == "bottom":
			self.box.bottomlabeling = self.labelstyle.lower()
			self.box.topticks = self.tickstyle.lower()
			self.box.bottomticks = self.tickstyle.lower()
		elif self.labeling <> None:
			raise Exception, "invalid side : "+str(self.labeling)+"\nAllowed are: left, right, top, bottom, None"
		self.box.addChangeListener(self)

	def plot(self, dev):
		dev.pushWorldMatrix(self.matrix)
		step = 1.0 / (self.colors - 1)

		#flat = Numeric.ravel(self.image.data)
		#immin, immax = min(flat), max(flat)
		datawidth = self.image.datamax - self.image.datamin
		datamin = (self.image.datamin - self.image.datamin) / datawidth
		datamax = (self.image.datamax - self.image.datamin) / datawidth
		#print "min/max", datamin, datamax
		#print self.image.datamin, self.image.datamax

		if self.direction == "up":
			data = Numeric.array([Numeric.arange(0, 1 + step/2, step)]*1)
			data = Numeric.transpose(data)
			corrmatrix = Matrix.translate((-0.5, 0.0))
		elif self.direction == "left":
			data = Numeric.array([Numeric.arange(1, 0 - step/2, -step)]*1)
			corrmatrix = Matrix.translate((0.0, -0.5))
		elif self.direction == "down":
			data = Numeric.array([Numeric.arange(1, 0 - step/2, -step)]*1)
			data = Numeric.transpose(data)
			corrmatrix = Matrix.translate((-0.5, 0.0))
		elif self.direction == "right":
			data = Numeric.array([Numeric.arange(0, 1 + step/2, step)]*1)
			corrmatrix = Matrix.translate((0.0, -0.5))
		else:
			raise Exception, "invalid direction : "+str(self.direction)+"\nAllowed are: left, right, up, down"

		height, width = data.shape
		#print "width=",width, data
		scalematrix = Matrix.scaleXY(1.0/width, 1.0/height)

		dev.pushWorldMatrix(corrmatrix * scalematrix)

		image = self.image
		dev.plotImage(data, colormap=image.colormap, datamin=datamin, datamax=datamax, function=image.function, origin=(0,0))
		dev.popWorldMatrix()


		if self.direction == "up":
			if self.sym == "symmetric":
				maxabs = max([abs(self.image.datamin),abs(self.image.datamax)])
				p1 = (0,-maxabs)
				p2 = (1,maxabs)
			else:	
				p1 = (0,self.image.datamin)
				p2 = (1,self.image.datamax)
		if self.direction == "down":
			if self.sym == "symmetric":
				maxabs = max([abs(self.image.datamin),abs(self.image.datamax)])
				p1 = (0,maxabs)
				p2 = (1,-maxabs)
			else:	
				p1 = (0,self.image.datamax)
				p2 = (1,self.image.datamin)
		if self.direction == "left":
			if self.sym == "symmetric":
				maxabs = max([abs(self.image.datamin),abs(self.image.datamax)])
				p1 = (maxabs,0)
				p2 = (-maxabs,1)
			else:	
				p1 = (self.image.datamax,0)
				p2 = (self.image.datamin,1)
		if self.direction == "right":
			if self.sym == "symmetric":
				maxabs = max([abs(self.image.datamin),abs(self.image.datamax)])
				p1 = (-maxabs,0)
				p2 = (maxabs,1)
			else:	
				p1 = (self.image.datamin,0)
				p2 = (self.image.datamax,1)

		x1, y1 = p1
		x2, y2 = p2
		worldmatrix = (Matrix.translate(p1) * Matrix.scaleXY(x2-x1, y2-y1)).inverse()
		dev.pushWorldMatrix(worldmatrix)
		self.box.plot(dev)
		dev.popWorldMatrix()

	def example():
		import Kplot

		print "Wedge example"
		print
		print 'The input of wedge is a Kplot.Image, so first create te data:'
		print 'row0 = range(10)'
		print 'row1 = range(10)'
		print 'row0.reverse() # reverse bottom row'
		print 'image = Kplot.Image([row0, row1])'
		row0 = range(10)
		row1 = range(10)
		row0.reverse() # reverse bottom row
		image = Kplot.Image([row0, row1])
		print
		print 'A wedge can then be created by:'
		print 'wedge = Kplot.Wedge(image, "right", "bottom"),'
		print 'where "right" is thge direction in which the wedge is filled'
		print 'and "bottom" the position of the labeling'
		print 
		wedge = Kplot.Wedge(image, "right", 'bottom')
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		wedgepanel, imagepanel = panel.split(0.2, "horizontal")
		wedgepanel.add(wedge)
		imagepanel.add(image)
		imagepanel.setWorldImage(image)
		dev.output()
		dev.close()

	def test():
		import Kplot
		dev = Kplot.createDevice()

		print "wedge should go to the direction from black to white"

		image = Kplot.Image([[1,2]], datamin=1.5, colormap=Kplot.ColorMap.blackwhite)
		panel = dev.createPanel()
		panels = panel.mosaic(2,2)
		panels[0][0].add(Wedge(image, "up"))
		panels[0][0].add(Text("up", (0.5, 0.5), color="white"))

		panels[0][1].add(Wedge(image, "right"))
		panels[0][1].add(Text("right", (0.5, 0.5), color="white"))

		panels[1][1].add(Wedge(image, "down"))
		panels[1][1].add(Text("down", (0.5, 0.5), color="white"))

		panels[1][0].add(Wedge(image, "left"))
		panels[1][0].add(Text("left", (0.5, 0.5), color="white"))

		dev.output()
		dev.close()


	example = staticmethod(example)
	test = staticmethod(test)
