import Kplot

class Histogram(object):
	"""Plots a simple histogram using binned data

	example:
		import Kplot
		data = [5,2,9,4,1,4,6,2]
		g = Kplot.simple.Histogram(data)

	Attributes of Graph:
	  device:      a Kplot.Device object
	  panel:       a Kplot.Panel object
	  histogram:   the histogram itself, a Kplot.Histogram object
	  box:         the grid and axes, a Kplot.Box object
	  toplabel:
	  leftlabel:
	  bottomlabel:
	  rightlabel: the four labels around the graph, bottomlabel has the text
	            "x", the leftlabel the text "y"

	example:
		g.toplabel.text.text = "f(x) = x ** 2"
		g.toplabel.text.color = "red"
		g.box.xinterval = 1
		g.box.drawgrid = False

	g.toplabel.text.text may look strange, but g.toplabel.text is a Kplot.Text
	object, where text is one of it's properties.

	To find information about a PlotObject, use <object>.printInfo()
	or use the help function. The help function will give a description of the
	class.

	example:
		g.box.printInfo()
		help(Kplot.Box)

	
	Default, the labels go from 0 to len(data)-1. The xaxis can be scaled
	using the xoffset argument. To make the labels appear in the middle of the
	bins, use an xoffset of 0.5.

	The next example will have the xaxis scaled, and the xoffset set to -2, so that
	the 0 starts in the middle.
	example:
		import Kplot
		data = [5,2,9,4,1,4,6,2]
		g = Kplot.simple.Histogram(data, xscale=4, xoffset=-2)

	If you want the 1 to appear in the middle, you also have to adjust the box.
	example:
		import Kplot
		data = [5,2,9,4,1,4,6,2]
		g = Kplot.simple.Histogram(data, xscale=3, xoffset=-0.5)
		g.box.xstart = 1


	Arguments:
	data -- the binned data
	xscale -- scales the x-axis(xscale > 1 makes the numbers at the labels bigger)
	xoffset -- translates the x-axis, so that it starts at xoffset(translation is
		done before scaling)

	"""
	def __init__(self, data, xscale=1.0, xoffset=0.0):
		self.device = Kplot.createDevice()
		self.device.setAutoReplot(False)
		self.panel = self.device.createPanel()
		self.histogram = Kplot.Histogram(data)
		self.box = Kplot.Box()
		self.toplabel = Kplot.Label("", "top")
		self.leftlabel = Kplot.Label("y", "left", displacement=15)
		self.bottomlabel = Kplot.Label("x", "bottom", displacement=15)
		self.rightlabel = Kplot.Label("", "right")

		self.rescale(xscale, xoffset)
		self.panel.add(self.box)
		self.panel.add(self.histogram)
		self.panel.add(self.toplabel)
		self.panel.add(self.bottomlabel)
		self.panel.add(self.leftlabel)
		self.panel.add(self.rightlabel)
		self.setWorld()
		self.device.setAutoReplot(True)


	def __del__(self):
		self.device.close()

	def setWorld(self):
		data = self.histogram.bindata
		p1 = (0, 0)
		p2 = (len(data),max(data))
		self.panel.setWorld(p1=p1, p2=p2)
		self.replot()

	def replot(self):
		"""Forces a replot"""
		self.device.output()
		
	def rescale(self, xscale, xoffset):
		"""Rescales the x axis"""
		self.device.setAutoReplot(False)
		matscale = Kplot.Matrix.translate((xoffset, 0))
		mattrans = Kplot.Matrix.scaleXY(xscale, 1)
		matrix =  (matscale * mattrans).inverse()
		self.box.xinterval = xscale
		self.box.matrix = matrix
		self.replot()
		self.device.setAutoReplot(True)
